package readxls;
/*-**********************************************************************
 * File: XlsHandler.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import readxls.control.LocaleInfo;
import readxls.type.CellTypeDefine;

/** Excel ファイル操作.
 * <P>
 * Excel ファイルを読み込むために依存ライブラリを使用する手順を
 * 実装するためのインタフェース。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public interface XlsHandler {

	/** 機能.
	 * @param localeInfo
	 * ロケール情報を設定する。
	 * @see LocaleInfo
	 */
	void setTimeZoneRawOffset(LocaleInfo localeInfo);

	/** 機能.
	 * <P>
	 * JExcelAPI は GMT での日時を日付型セルの入力値としているので、
	 * それをロケール情報のタイムゾーンに補正する。
	 * <BR>
	 * Apatch POI ではそのような補正は不要で、POI から取得した入力値
	 * そのままでよいと思われる。
	 * </P>
	 * @param value
	 * 日付型セルの入力値
	 * @return
	 * ロケール情報のタイムゾーンに補正した日付
	 * @see LocaleInfo
	 */
	long adjustToTimeZone(final Date value);

	/** 機能.
	 * <P>
	 * すでに Excel ファイルをワークブックに取り込んだかどうかの状態を
	 * 返す。
	 * </P>
	 * @return
	 * すでに Excel ファイルを取り込んでいれば true
	 * を、そうでなければ false を返す。
	 */
	boolean isLoadedWorkbook();

	/** 機能.
	 * <P>
	 * Excel ファイルを取り込んで、
	 * 依存ライブラリのワークブックを生成する。
	 * </P>
	 * @param is
	 * Excel ファイルの入力元ストリーム
	 * @param localeInfo
	 * ロケール情報
	 * @param isOOXML
	 * Excel ファイルが Office Open XML (.xlsx) 形式であれば true
	 * を指定する。
	 * @return
	 * Excle ファイルの取り込みに成功すれば true
	 * を。そうでなければ false を返す。
	 * @throws ReadXlsException
	 * 取り込み処理中に異常がおれば発生する。
	 * @throws IOException
	 * 入出力エラーがおきれば発生する。
	 */
	boolean loadWorkbook(InputStream is,
							LocaleInfo localeInfo,
							boolean isOOXML)
									throws ReadXlsException, IOException;

	/** 機能.
	 * <P>
	 * 指定された名称のシートをワークブックから取得し、それを保持する。
	 * </P>
	 * @param sheetName
	 * シートの名称
	 * @return
	 * シートが取得できれば true を、そうでなければ false を返す。
	 */
	boolean setSheet(String sheetName);

	/** 機能.
	 * @return
	 * シートに入力さている行数 (0～) を返す、
	 */
	int getLastRowNum();

	/** 機能.
	 * <P>
	 * 指定された行のセルを読み込んで保持する。
	 * <BR>
	 * 行番号がゼロであれば要素数ゼロのセル配列を保持する。
	 * </P>
	 * @param rowNo
	 * 読み込む行の行番号 (1～)
	 */
	void readInCells(final int rowNo);

	/** 機能.
	 * @return
	 * {@link XlsHandler#readInCells(int)}
	 * で読み込んだ行に入力されている列数を返す。
	 */
	int lengthOfLastReadCells();

	/** 機能.
	 * <P>
	 * {@link XlsHandler#readInCells(int)}
	 * で読み込んだ行から、指定された列のセルを読み込んで保持する。
	 * <BR>
	 * 行番号にゼロを指定すれば null を保持する。
	 * </P>
	 * @param columnNo
	 * 読み込む列の列番号 (0,1～)
	 */
	void readInLastReadCell(int columnNo);

	/** 機能.
	 * @return
	 * {@link XlsHandler#readInLastReadCell(int)}
	 * で読み込んだセルの型を返す。
	 * @see CellTypeDefine
	 */
	CellTypeDefine getCellType();

	/** 機能.
	 * @return
	 * 論理値型のセルの入力値を返す。
	 */
	boolean getBooleanCellValue();

	/** 機能.
	 * @return
	 * 日付型のセルの入力値を返す。
	 */
	Date getDateCellValue();

	/** 機能.
	 * @return
	 * 数値型のセルの入力値を返す。
	 */
	double getNumberCellValue();

	/** 機能.
	 * @return
	 * 文字列型のセルの入力値を返す。
	 */
	String getStringCellValue();

	/** 機能.
	 * <P>
	 * 指定された行が読み込みを停止する行であるかどうかを判定するために、
	 * その行のセルを読み込み保持する。
	 * </P>
	 * @param rowNo
	 * 読み込む行の行番号 (1～)
	 */
	void readInBreakerCells(int rowNo);

	/** 機能.
	 * @return
	 * {@link XlsHandler#readInBreakerCells(int)}
	 * で読み込んだ行に入力されている列数を返す。
	 */
	int getBreakerCellsLength();

	/** 機能.
	 * <P>
	 * {@link XlsHandler#readInBreakerCells(int)}
	 * で読み込んだ行から、指定された列番号 (1～)
	 * のセルを読み込み保持する、
	 * </P>
	 * @param columnNo
	 * 読み込む列の列番号 (1～)
	 */
	void readInBreakerCell(int columnNo);

	/** 機能.
	 * <P>
	 * {@link XlsHandler#readInBreakerCell(int)}
	 * で読み込んだセルが空セルであるかどうかを判定する。
	 * </P>
	 * @return
	 * 空セルであれば true を、そうでなければ false を返す。
	 */
	boolean isBreakerCellEmpty();

	/** 機能.
	 * <P>
	 * Excel ファイルをロードしたワークブック、シートなどを解放する。
	 * </P>
	 */
	void close();

}
