package readxls.control;
/*-**********************************************************************
 * File: CellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import readxls.InvalidCellInfoCollector;
import readxls.ReadXls;
import readxls.ReadXlsException;
import readxls.XlsHandler;
import readxls.type.CellTypeDefine;
import readxls.type.DateCellDefine;
import readxls.type.EmptyCellDefine;
import readxls.type.NumberCellDefine;
import readxls.validate.Checker;
import readxls.validate.DurationChecker;
import readxls.validate.RangeChecker;

/** 読み込みセルの項目定義.
 * <BR>
 * 読み込むセルの型に即した定義を行うクラスの基底クラス。
 * <BR>
 * この仮想クラスを拡張するクラスすべてで指定できる定義情報は以下のとおり。
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須であるときに未入力だった場合のメッセージ
 * </LI>
 * <LI>
 * セルの型が項目定義と異なる場合のメッセージ
 * </LI>
 * </UL>
 * <P>
 * {@link HeaderDefine}
 * と
 * {@link DetailDefine}
 * には、定義情報の正当性を確認する機能
 * {@link CellDefine#isValidParameters(XlsHandler, LocaleInfo)}
 * を提供している。
 * <BR>
 * この機能は、セルの型に即した継承クラスそれぞれの正当性を確認する機能
 * {@link CellDefine#checkParameters(XlsHandler, LocaleInfo)}
 * を利用している。
 * </P>
 * <P>
 * {@link ReadXls}
 * には、セルの入力値を読み込み、その値が定義情報で指定された値域にあること
 * (文字列長が超過しないこと、使用できる文字種、数値が上限と下限に収まっていること)
 * を確認した上で、文字列に変換する機能
 * {@link CellDefine#readCell(int, int, XlsHandler, InvalidCellInfoCollector)}
 * を提供している。
 * <BR>
 * この機能は、セルの型に即した継承クラスそれぞれの正当性を確認する機能
 * {@link CellDefine#checkCellType(CellTypeDefine)}
 * 、
 * {@link CellDefine#isValidCellValue(XlsHandler, StringBuilder)}
 * と
 * {@link CellDefine#convertCellValue(XlsHandler)}
 * を利用している。
 * </P>
 * <P>
 * また、日付型と数値型のセルの入力値が特定の範囲内にあることを
 * 確認するために、
 * {@link DurationChecker} と {@link RangeChecker}
 * を、
 * {@link DateCellDefine} と {@link NumberCellDefine}
 * で指定できる。
 * </P>
 * @see EmptyCellDefine
 * @see ReadXls
 * @see XlsHandler
 * @see LocaleInfo
 * @see HeaderDefine
 * @see DetailDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public abstract class CellDefine {

	/** 項目定義情報.
	 * <P>
	 * 読み込むセルの列番号 (1 ～)
	 * </P>
	 */
	private int columnNo = -1;

	/** 項目定義情報.
	 * <P>
	 * 項目名
	 * </P>
	 */
	private String name = null;

	/** 項目定義情報.
	 * <P>
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * </P>
	 */
	private String lostMandatoryMessageID = null;

	/** 項目定義情報.
	 * <P>
	 * 列番号のセルが論理値型でなかったことを示すメッセージ
	 * </P>
	 */
	private String invalidTypeMessageId = null;

	/** 項目定義情報.
	 * <P>
	 * セルの入力値が特定の範囲内にあることを確認するための
	 * {@link Checker} 。
	 * </P>
	 */
	private Checker checker = null;

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#columnNo}
	 * の getter 。
	 * </P>
	 * @return CellDefine#columnNo の設定値
	 */
	public final int getColumnNo() {
		return this.columnNo;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#columnNo}
	 * の setter 。
	 * </P>
	 * @param columnNo CellDefine#columnNo に設定する値
	 */
	public final void setColumnNo(final int columnNo) {
		this.columnNo = columnNo;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#name}
	 * の getter 。
	 * </P>
	 * @return CellDefine#name の設定値
	 */
	public final String getName() {
		return this.name;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#name}
	 * の setter 。
	 * </P>
	 * @param name CellDefine#name に設定する値
	 */
	public final void setName(final String name) {
		this.name = name;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#lostMandatoryMessageID}
	 * の getter 。
	 * </P>
	 * @return CellDefine#lostMandatoryMessageID の設定値
	 */
	public final String getLostMandatoryMessageID() {
		return this.lostMandatoryMessageID;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#lostMandatoryMessageID}
	 * の setter 。
	 * </P>
	 * @param lostMandatoryMessageID
	 * CellDefine#lostMandatoryMessageID に設定する値
	 */
	public final void setLostMandatoryMessageID(
									final String lostMandatoryMessageID) {
		this.lostMandatoryMessageID = lostMandatoryMessageID;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#invalidTypeMessageId}
	 * の getter 。
	 * </P>
	 * @return CellDefine#invalidTypeMessageId の設定値
	 */
	public final String getInvalidTypeMessageId() {
		return this.invalidTypeMessageId;
	}

	/** getter/setter.
	 * <P>
	 * {@link CellDefine#invalidTypeMessageId}
	 * の setter 。
	 * </P>
	 * @param invalidTypeMessageId
	 * CellDefine#invalidTypeMessageId に設定する値
	 */
	public final void setInvalidTypeMessageId(
									final String invalidTypeMessageId) {
		this.invalidTypeMessageId = invalidTypeMessageId;
	}

	/** getter/setter.
	 * @return
	 * {@link CellDefine#checker} を返す。
	 */
	public final Checker getChecker() {
		return checker;
	}

	/** getter/setter.
	 * @param checkers
	 * {@link CellDefine#checker} に設定する {@link Checker}
	 */
	public final void setChecker(final Checker checkers) {
		this.checker = checkers;
	}

	/** コンストラクタ.
	 * <P>
	 * セルの型に即した項目を定義するクラスのコンストラクタの基底メソッド。
	 * <BR>
	 * {@link CellDefine}
	 * を継承するクラスは、それぞれに固有のフィールド値も引数に取るように
	 * コンストラクタを実装する。
	 * <BR>
	 * その実装したコンストラクタから、この基底コンストラクタを呼び出す。
	 * </P>
	 *
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが論理値型でなかったことを示すメッセージ
	 * @param checker
	 * セルの入力値の正当性を確認する手順 {@link Checker} のリスト
	 */
	public CellDefine(final int			columnNo,
						final String	name,
						final String	lostMandatoryMessageID,
						final String	invalidTypeMessageId,
						final Checker	checker) {
		this.columnNo				= columnNo;
		this.name					= name;
		this.lostMandatoryMessageID	= lostMandatoryMessageID;
		this.invalidTypeMessageId	= invalidTypeMessageId;
		this.checker = checker;
	}

	/** 機能.
	 * <P>
	 * {@link HeaderDefine}
	 * と
	 * {@link DetailDefine}
	 * に、定義情報の正当性を確認する機能を提供する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 以下のいずれかの条件であったら false
	 * を返す。そうでなければ項目設定は正常と判断し true
	 * を返す。
	 * <OL>
	 * <LI>
	 * 列番号がゼロか負数。
	 * </LI>
	 * <LI>
	 * セルの方が空白セル
	 * {@link EmptyCellDefine}
	 * ではなく、項目名と型不一致時のメッセージが null か空文字列 "" 。
	 * </LI>
	 * <LI>
	 * セル型に即した項目定義クラスの項目定義確認メソッド
	 * {@link CellDefine#checkParameters(XlsHandler, LocaleInfo)}
	 * の判定結果が不正 false となった。
	 * </LI>
	 * </OL>
	 */
	public final boolean isValidParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		return	getColumnNo() > 0
				&& (this instanceof EmptyCellDefine
					|| (getName() != null
						&& !"".equals(getName())
						&& getInvalidTypeMessageId() != null
						&& !"".equals(getInvalidTypeMessageId())
						)
					)
				&& checkParameters(xlsHandler, localeInfo);
	}

	/** ユーティリティ.
	 * <P>
	 * セル型に即した項目定義クラスの読み込むセルの型と値域
	 * (文字数、文字種、数値の定義域)
	 * のチェック機能でセルの入力値を確認してから文字列に変換する。
	 * @param rowNo
	 * 読み込むセルの行番号 (1 ～)
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * セル入力値の確認で不正なデータが検知された場合ごとのメッセージが
	 * 出力される。
	 * @return
	 * セル入力値が正当なものであればそれを文字列に変化した結果を、
	 * 不正が検知されれば null を返す。
	 * @throws ReadXlsException
	 * 不正なセル入力値が検知された回数が上限を超えた場合などの
	 * なんらかの障害が発生した場合に発生する。
	 */
	private String convert(final int			rowNo,
							final int			columnNo,
							final XlsHandler	xlsHandler,
							final StringBuilder	msgIdSb)
									throws ReadXlsException {
		String value = null;
		if (isValidCellValue(xlsHandler, msgIdSb)) {
			value = convertCellValue(xlsHandler);
		}
		return value;
	}

	/** 機能.
	 * <P>
	 * {@link ReadXls}
	 * に、セルの入力値を読み込み、
	 * その値が定義情報で指定された値域にあること
	 * (文字列長が超過しないこと、
	 * 使用できる文字種、
	 * 数値が上限と下限に収まっていること)
	 * を確認した上で文字列に変換する機能を提供する。
	 * </P>
	 * @param rowNo
	 * 読み込むセルの行番号 (1 ～)
	 * @param colNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param ic
	 * 不正な入力値であるセルの情報を通知するエラーコレクタ
	 * {@link InvalidCellInfoCollector} の実装
	 * @return
	 * 不正な入力値であるセルであった場合か未入力なセルの場合は空文字列 ""
	 * を、そうでなｋればセルの入力値を文字列に変換した結果を返す。
	 * @throws ReadXlsException
	 * 不正なセル入力値が検知された回数が上限を超えた場合などの
	 * なんらかの障害が発生した場合に発生する。
	 */
	public final String readCell(
							final int						rowNo,
							final int						colNo,
							final XlsHandler				xlsHandler,
							final InvalidCellInfoCollector	ic)
												throws ReadXlsException {
		String			result		= ReadXls.EMPTY_VALUE;
		String			msgId		= null;
		StringBuilder	msgIdSb		= new StringBuilder();
		CellTypeDefine	cellType	= xlsHandler.getCellType();
		if (CellTypeDefine.EMPTY == cellType) {
			msgId = getLostMandatoryMessageID();
			if (msgId == null || "".equalsIgnoreCase(msgId)) {
				msgId = null;
			}
		}
		else if (!checkCellType(cellType)) {
			msgId = getInvalidTypeMessageId();
		}
		else if ((result = convert(rowNo,
									colNo,
									xlsHandler,
									msgIdSb)) == null) {
			msgId = (msgIdSb.length() > 0)
					?	msgIdSb.toString()
					:	null;
		}
		if (msgId != null) {
			ic.notifyInvalidCellInfo(rowNo, colNo, this, cellType, msgId);
		}
		return result;
	}

	/** 継承クラスで実装するメソッド.
	 * <P>
	 * セル型に即した項目定義クラスそれぞれの項目定義パラメータ値の正当性を
	 * 確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 正当な項目定義パラメータであれば true
	 * を、そうでなければ false を返す。
	 */
	protected abstract boolean checkParameters(XlsHandler	xlsHandler,
												LocaleInfo	localeInfo);

	/** 継承クラスで実装するメソッド.
	 * <P>
	 * セル型に即した項目定義クラスそれぞれで読み込むセルの型を確認する。
	 * </P>
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * セルの型が読み込むセルと一致していれば true
	 * を、そうでなければ false を返す。
	 */
	protected abstract boolean checkCellType(CellTypeDefine cellType);

	/** 継承クラスで実装するメソッド.
	 * <P>
	 * セル型に即した項目定義クラスそれぞれで読み込むセルの入力値の値域を
	 * 確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 確認結果が正当であれば true を、そうでなければ false を返す。
	 * @throws ReadXlsException
	 * 入力値の確認で障害が発生した場合に発生する。
	 */
	protected abstract boolean isValidCellValue(XlsHandler		xlsHandler,
												StringBuilder	msgIdSb)
											throws ReadXlsException;

	/** 継承クラスで実装するメソッド.
	 * <P>
	 * セル型に即した項目定義クラスそれぞれで読み込むセルの入力値を
	 * 文字列に変換する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * セルから読み込んだ入力値を文字列に変換した結果
	 */
	protected abstract String convertCellValue(XlsHandler	xlsHandler);

}
