package readxls.control;
/*-**********************************************************************
 * File: LocaleInfo.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.nio.charset.Charset;
import java.util.Locale;
import java.util.TimeZone;

/** ロケール情報.
 * <P>
 * ロケールに関する情報を設定する。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class LocaleInfo {

	/** ISO 言語コード ISO-639 で定義される 2 桁の小文字です (日本語は "ja").
	 * <BR>
	 * <A HREF="http://www.loc.gov/standards/iso639-2/englangn.html">
	 * Codes for the representation of names of languages (Library of Congress)
	 * </A>
	 * @see java.util.Locale
	 */
	public static final String LOCALE_NAME = "ja";


	/** タイムゾーンをあらわす文字列です (日本は "Japan").
	 * @see TimeZone
	 */
	public static final String TIME_ZONE_ID = "Japan";

	/** 文字列型セルの入力文字列が最大入力長を超えたことを確認するために文字コード変換するキャラセット名 (シフト JIS は "MS932").
	 * @see Charset
	 */
	public static final String CHARSET_TO_GETBYTES = "MS932";

	/** JExcelAPI を使用する際に、ワークブックの文字コードを指定する (シフト JIS は "windows-31j").
	 * @see Charset
	 */
	public static final String EXCODE_OF_EXL_WORKBOOK = "windows-31j";

	/** ロケール.
	 * <P>
	 * {@link LocaleInfo#localeName} で指定されたロケール名（言語）に対応するロケール。
	 * </P>
	 */
	private Locale locale = new Locale.Builder()
									.setLanguage(LocaleInfo.LOCALE_NAME)
									.build();

	/** ロケール情報.
	 * <P>
	 * ロケールを指定する。
	 * 既定値は
	 * {@link LocaleInfo#LOCALE_NAME}
	 * </P>
	 */
	private String localeName = LocaleInfo.LOCALE_NAME;

	/** ロケール情報.
	 * <P>
	 * タイムゾーンを指定する。
	 * 既定値は
	 * {@link LocaleInfo#TIME_ZONE_ID}
	 * </P>
	 */
	private String timeZoneId = LocaleInfo.TIME_ZONE_ID;

	/** ロケール情報.
	 * <P>
	 * 文字列型セルの入力文字列を変換する文字コードを指定する。
	 * 既定値は
	 * {@link LocaleInfo#CHARSET_TO_GETBYTES}
	 * </P>
	 */
	private String charsetToGetbytes = LocaleInfo.CHARSET_TO_GETBYTES;

	/** ロケール情報.
	 * <P>
	 * JExcelAPI を使用する際の、ワークブックの文字コードを指定する。
	 * 既定値は
	 * {@link LocaleInfo#EXCODE_OF_EXL_WORKBOOK}
	 * </P>
	 */
	private String excodeOfExlWorkbook = LocaleInfo.EXCODE_OF_EXL_WORKBOOK;

	/** getter/setter.
	 * @return
	 * {@link LocaleInfo#locale} を返す。
	 */
	public final Locale getLocale() {
		return this.locale;
	}

	/** getter/setter.
	 * @param locale
	 * {@link LocaleInfo#locale} に指定するロケール
	 */
	public final void setLocale(final Locale locale) {
		this.locale = locale;
	}

	/** getter/setter.
	 * @return
	 * {@link LocaleInfo#localeName}
	 * を返す。
	 */
	public final String getLocaleName() {
		return this.localeName;
	}

	/** getter/setter.
	 * @param localName
	 * {@link LocaleInfo#localeName}
	 * に設定する値。
	 */
	public final void setLocaleName(final String localName) {
		this.localeName = localName;
		setLocale(new Locale.Builder().setLanguage(localName).build());
	}

	/** getter/setter.
	 * @return
	 * {@link LocaleInfo#timeZoneId}
	 * を返す。
	 */
	public final String getTimeZoneId() {
		return this.timeZoneId;
	}

	/** getter/setter.
	 * @param timeZoneId
	 * {@link LocaleInfo#timeZoneId}
	 * に設定する値。
	 */
	public final void setTimeZoneId(final String timeZoneId) {
		this.timeZoneId = timeZoneId;
	}

	/** getter/setter.
	 * @return
	 * {@link LocaleInfo#charsetToGetbytes}
	 * を返す。
	 */
	public final String getCharsetToGetbytes() {
		return this.charsetToGetbytes;
	}

	/** getter/setter.
	 * @param charsetToGetbytes
	 * {@link LocaleInfo#charsetToGetbytes}
	 * に設定する値。
	 */
	public final void setCharsetToGetbytes(
										final String charsetToGetbytes) {
		this.charsetToGetbytes = charsetToGetbytes;
	}

	/** getter/setter.
	 * @return
	 * {@link LocaleInfo#excodeOfExlWorkbook}
	 * を返す。
	 */
	public final String getExcodeOfExlWorkbook() {
		return this.excodeOfExlWorkbook;
	}

	/** getter/setter.
	 * @param excodeOfExlWorkbook
	 * {@link LocaleInfo#excodeOfExlWorkbook}
	 * に設定する値。
	 */
	public final void setExcodeOfExlWorkbook(
										final String excodeOfExlWorkbook) {
		this.excodeOfExlWorkbook = excodeOfExlWorkbook;
	}

	/** コンストラクタ.
	 * @param localName
	 * ロケール名
	 * {@link LocaleInfo#localeName}
	 * に設定する値。
	 * @param timeZoneId
	 * タイムゾーン
	 * {@link LocaleInfo#timeZoneId}
	 * に設定する値。
	 * @param charsetToGetbytes
	 * 入力文字列を変換する文字コード
	 * {@link LocaleInfo#charsetToGetbytes}
	 * に設定する値。
	 * @param excodeOfExlWorkbook
	 * JExcelAPI を使用する場合に、ワークブックの文字コード
	 * {@link LocaleInfo#excodeOfExlWorkbook}
	 * に設定する値。
	 */
	public LocaleInfo(final String		localName,
						final String	timeZoneId,
						final String	charsetToGetbytes,
						final String	excodeOfExlWorkbook) {
		this.localeName				= localName;
		this.timeZoneId				= timeZoneId;
		this.charsetToGetbytes		= charsetToGetbytes;
		this.excodeOfExlWorkbook	= excodeOfExlWorkbook;
	}

	/** コンストラクタ.
	 * 日本語ロケールを設定する。
	 */
	public LocaleInfo() { }

	/** 機能.
	 * <P>
	 * ロケール名、タイムゾーン、変換文字コード、ワークブック文字コードが
	 * null や空文字列 "" ではないことを確認する。
	 * </P>
	 * @return
	 * ロケール情報の設定が正当であれば true
	 * を、そうでなければ false を返す。
	 */
	public final boolean isValid() {
		return	getLocaleName() != null
				&& getLocaleName().length() != 0
				&& getLocale() != null
				&& getTimeZoneId() != null
				&& getTimeZoneId().length() != 0
				&& getCharsetToGetbytes() != null
				&& getCharsetToGetbytes().length() != 0
				&& getExcodeOfExlWorkbook() != null
				&& getExcodeOfExlWorkbook().length() != 0;
	}

}
