package readxls.type;
/*-**********************************************************************
 * File: DateCellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.text.SimpleDateFormat;
import java.util.Date;

import readxls.XlsHandler;
import readxls.control.CellDefine;
import readxls.control.LocaleInfo;
import readxls.validate.Checker;
import readxls.validate.DurationChecker;


/** 読み込みセルの項目定義.
 * <P>
 * 日付型のセルを読み込むための項目定義を行う。
 * <BR>
 * 指定する定義情報は以下のとおり。
 * </P>
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須であるときに未入力だった場合のメッセージ
 * </LI>
 * <LI>
 * セルの型が項目定義と異なる場合のメッセージ
 * </LI>
 * <LI>
 * 日付型セルの入力値を文字列に変換するため
 * {@link SimpleDateFormat}
 * に適用するフォーマット文字列
 * </LI>
 * </UL>
 * @see CellDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class DateCellDefine extends CellDefine {

	/** フィールド.
	 * <P>
	 * 日付型セルの入力値を文字列に変換するための
	 * {@link SimpleDateFormat}
	 * </P>
	 */
	private SimpleDateFormat sdf = null;

	/** 項目定義情報.
	 * <P>
	 * 日付型セルの入力値を文字列に変換するための
	 * {@link DateCellDefine#sdf}
	 * に適用するフォーマット文字列
	 * </P>
	 */
	private String simpleDateFormat = null;

	/** getter/setter.
	 * <BR>
	 * {@link DateCellDefine#sdf}
	 * の getter 。
	 * @return DateCellDefine#sdf の値
	 */
	public final SimpleDateFormat getSdf() {
		return this.sdf;
	}

	/** getter/setter.
	 * <BR>
	 * {@link DateCellDefine#sdf}
	 * の setter 。
	 * @param sdf DateCellDefine#sdf に設定する値
	 */
	private void setSdf(final SimpleDateFormat sdf) {
		this.sdf = sdf;
	}

	/** getter/setter.
	 * <BR>
	 * {@link DateCellDefine#simpleDateFormat}
	 * の getter 。
	 * @return DateCellDefine#simpleDateFormat の値
	 */
	public final String getSimpleDateFormat() {
		return this.simpleDateFormat;
	}

	/** ユーティリティ.
	 * <P>
	 * {@link DateCellDefine#simpleDateFormat}
	 * に設定されている日付変換フォーマット文字列を
	 * {@link SimpleDateFormat}
	 * に適用したフォーマッタを
	 * {@link DateCellDefine#sdf}
	 * に設定する。
	 * </P>
	 */
	private void makeSdf() {
		String pattern = getSimpleDateFormat();
		if (pattern != null && !"".equalsIgnoreCase(pattern)) {
			setSdf(new SimpleDateFormat(pattern));
		}
	}

	/** getter/setter.
	 * <BR>
	 * {@link DateCellDefine#simpleDateFormat}
	 * の setter 。
	 * <P>
	 * {@link DateCellDefine#makeSdf}
	 * で
	 * {@link DateCellDefine#sdf}
	 * も設定する。
	 * </P>
	 * @param simpleDateFormat
	 * {@link DateCellDefine#simpleDateFormat} に設定する値
	 */
	public final void setSimpleDateFormat(final String simpleDateFormat) {
		this.simpleDateFormat = simpleDateFormat;
		makeSdf();
	}

	/** コンストラクタ.
	 * <P>
	 * 日付型セルの入力値を文字列に変換するための
	 * {@link SimpleDateFormat}
	 * に適用するフォーマット文字列を指定する。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが日付型でなかったことを示すメッセージ
	 * @param checker
	 * 日付型セルの入力値をとり、その正当性を確認する手順
	 * {@link DurationChecker}
	 * @param simpleDateFormat
	 * 入力値を文字列変換するためのフォーマット文字列
	 */
	public DateCellDefine(final int				columnNo,
							final String		name,
							final String		lostMandatoryMessageID,
							final String		invalidTypeMessageId,
							final Checker	checker,
							final String		simpleDateFormat) {
		super(columnNo,
				name,
				lostMandatoryMessageID,
				invalidTypeMessageId,
				checker);
		setSimpleDateFormat(simpleDateFormat);
	}

	/** 機能.
	 * <P>
	 * 日付型セルの文字列変換フォーマット
	 * {@link DateCellDefine#simpleDateFormat}
	 * が指定されていることを確認する。
	 * </P>
	 * <P>
	 * 日付型セルの文字列変換フォーマットが定義されていれば
	 * {@link XlsHandler#setTimeZoneRawOffset}
	 * でタイムゾーンのオフセット値 (GMT からの時差時間) を設定する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 日付型セルの文字列変換フォーマットが定義されていれば
	 * true を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		boolean result = false;
		if (getSimpleDateFormat() != null && getSdf() != null) {
			xlsHandler.setTimeZoneRawOffset(localeInfo);
			result = true;
		}
		if (getChecker() != null
			&& !(getChecker() instanceof DurationChecker)) {
			result = false;
		}
		return result;
	}

	/** 機能.
	 * <P>
	 * 読み込むセルの型が日付型
	 * {@link CellTypeDefine#DATE}
	 * であることを確認する。
	 * </P>
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * 読み込むセルの型が日付型であれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkCellType(final CellTypeDefine cellType) {
		return	CellTypeDefine.DATE == cellType;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 常に true を返す。
	 */
	@Override
	protected final boolean isValidCellValue(final XlsHandler	xlsHandler,
											final StringBuilder	msgIdSb) {
		boolean result = getChecker() == null
							|| ((DurationChecker)getChecker())
													.check(xlsHandler);
		if (!result) {
			msgIdSb.append(((DurationChecker)getChecker()).getMessage());
		}
		return result;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * 読み込みセルの入力値を、
	 * タイムゾーンに応じた日時で文字列に変換した結果。
	 * @see XlsHandler#getDateCellValue()
	 * @see XlsHandler#adjustToTimeZone(Date)
	 * @see SimpleDateFormat#format(Date)
	 */
	@Override
	protected final String convertCellValue(final XlsHandler xlsHandler) {
		Date value = xlsHandler.getDateCellValue();
		value.setTime(xlsHandler.adjustToTimeZone(value));
		return getSdf().format(value);
	}

}
