package readxls.validate;
/*-**********************************************************************
 * File: RangeChecker.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.math.BigDecimal;

import readxls.XlsHandler;

/** 読み込みセルの入力確認.
 * <P>
 * 数値型セルの入力値の範囲をチェックする。
 * </P>
 * @see Checker
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class RangeChecker implements Checker {

	/** 属性.
	 * <P>
	 * 数値範囲の最小値
	 * </P>
	 */
	private BigDecimal minimum = null;

	/** 属性.
	 * <P>
	 * 数値範囲の最大値
	 * </P>
	 */
	private BigDecimal maximum = null;

	/** 属性.
	 * <P>
	 * セルの入力値が範囲外の数値だった場合のメッセージ。
	 * </P>
	 */
	private String message = null;

	/** getter/setter.
	 * @return
	 * {@link RangeChecker#minimum} を返す。
	 */
	private BigDecimal getMinimum() {
		return minimum;
	}

	/** getter/setter.
	 * @param minimum
	 * {@link RangeChecker#minimum} に設定する値
	 */
	private void setMinimum(final BigDecimal minimum) {
		this.minimum = minimum;
	}

	/** getter/setter.
	 * @return
	 * {@link RangeChecker#maximum} を返す。
	 */
	private BigDecimal getMaximum() {
		return maximum;
	}

	/** getter/setter.
	 * @param maximum
	 * {@link RangeChecker#maximum} に設定する値
	 */
	private void setMaximum(final BigDecimal maximum) {
		this.maximum = maximum;
	}

	/** getter/setter.
	 * @return
	 * {@link RangeChecker#message} を返す。
	 */
	public final String getMessage() {
		return message;
	}

	/** getter/setter.
	 * @param message
	 * {@link RangeChecker#message} に設定するメッセージ。
	 */
	private void setMessage(final String message) {
		this.message = message;
	}

	/** コンストラクタ.
	 * <P>
	 * 数値型セルの入力値の範囲を確認する {@link Checker}。
	 * </P>
	 * @param minimum
	 * {@link RangeChecker#minimum} に設定する値
	 * @param maximum
	 * {@link RangeChecker#maximum} に設定する値
	 * @param message
	 * 入力値が範囲外だった場合のメッセージ
	 */
	public RangeChecker(final BigDecimal	minimum,
						final BigDecimal	maximum,
						final String		message) {
		this.setMinimum(minimum);
		this.setMaximum(maximum);
		this.setMessage(message);
	}

	/** 機能.
	 * <P>
	 * 数値型セルの入力値が {@link RangeChecker#minimum} 以上、
	 * {@link RangeChecker#maximum} 以下であることを確認する 。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * 入力された数値が範囲内であれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	public final boolean check(final XlsHandler xlsHandler) {
		return (getMinimum() == null
					|| getMinimum().compareTo(
						BigDecimal.valueOf(
								xlsHandler.getNumberCellValue())) <= 0)
				&& (getMaximum() == null
					|| getMaximum().compareTo(
						BigDecimal.valueOf(
								xlsHandler.getNumberCellValue())) >= 0);
	}

}
