package testreadxls;
/*-**********************************************************************
 * File: TestUsingJxl.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements a program to test "readxls"
 *  package, using JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import readxls.InvalidCellInfo;
import readxls.InvalidCellInfoCollector;
import readxls.ReadXls;
import readxls.ReadXlsException;
import readxls.ReadXlsUtil;
import readxls.ReturnCode;
import readxls.XlsHandler;
import readxls.control.Breaker;
import readxls.control.CellDefine;
import readxls.control.DetailCallable;
import readxls.control.DetailDefine;
import readxls.control.HeaderCallable;
import readxls.control.HeaderDefine;
import readxls.control.LocaleInfo;
import readxls.control.RowColumnBreaker;
import readxls.type.BooleanCellDefine;
import readxls.type.CellTypeDefine;
import readxls.type.DateCellDefine;
import readxls.type.EmptyCellDefine;
import readxls.type.NumberCellDefine;
import readxls.type.StringCellDefine;
import readxls.validate.DurationChecker;
import readxls.validate.RangeChecker;
import readxlsforjxl.XlsHandleForJxl;
import readxlsforpoi.XlsHandleForPoi;

/** テストプログラム概要.
 * <BR>
 * {@link ReadXls} を、JExcelAPI を使用してテストする Java プログラム。
 * <BR>
 * (<A HREF="http://jexcelapi.sourceforge.net/">
 * JExcelApi - A Java API to read, write, and modify Excel spreadsheets)
 * </A>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public final class TestUsingJxl implements HeaderCallable,
											DetailCallable,
											InvalidCellInfoCollector {

	/** コンストラクタ.
	 * <BR>
	 * テストプログラムの手順を実装しているインスタンスを生成している。
	 */
	private TestUsingJxl() {}

	/** 出力情報のチェック手順.
	 * <BR>
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * と
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだヘッダ情報の内容が一致していることを確認している。
	 *
	 * @param headersA 読み込んだヘッダ情報の一方
	 * @param headersB 読み込んだヘッダ情報のもう一方
	 * @return
	 * ヘッダ情報の内容が一致していれば true
	 * を、そうでなければ false を返す。
	 */
	private boolean checkHeaders(
							final Hashtable<String, String>	headersA,
							final Hashtable<String, String>	headersB) {
		boolean result = true;
		if (headersA != null && headersB != null) {
			Enumeration<String> keys = headersA.keys();
			while (keys.hasMoreElements()) {
				String key = keys.nextElement();
				if (!headersB.containsKey(key)
				|| !headersA.get(key).equals(headersB.get(key))) {
					System.out.println(
						"!!!ERROR!!! "
						+ "ヘッダ情報の読み込み内容が項目名 \"" + key + "\""
						+ "で一致していません。"
						);
					System.out.flush();
					result = false;
				}
			}
		}
		return result;
	}

	/** 出力情報のチェック手順.
	 * <BR>
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * と
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだ明細情報の内容が一致することを確認している。
	 *
	 * @param detailsA 読み込んだ明細情報の一方
	 * @param detailsB 読み込んだ明細情報のもう一方
	 * @return
	 * 明細情報の内容が一致していれば true
	 * を、そうでなければ false を返す。
	 */
	private boolean checkDetails(
								final List<List<String>>	detailsA,
								final List<List<String>>	detailsB) {
		boolean result = true;
		if (detailsA != null && detailsB != null) {
			if (detailsA.size() != detailsB.size()) {
				System.out.println(
						"!!!ERROR!!! "
						+ "明細情報の読み込み内容の行数が一致しません。"
						);
				System.out.flush();
				result = false;
			}
			else {
				for (int r = 0; r < detailsA.size(); ++r) {
					List<String> rowA = detailsA.get(r);
					List<String> rowB = detailsB.get(r);
					if (rowA.size() != rowB.size()) {
						System.out.println(
								"!!!ERROR!!! "
								+ "明細情報の読み込み内容の"
								+ (r + 1) + "行目の列数が一致しません。"
								);
						System.out.flush();
						result = false;
					}
					else {
						for (int c = 0; c < rowA.size(); ++c) {
							if (!rowA.get(c).equals(rowB.get(c))) {
								System.out.println(
										"!!!ERROR!!! "
										+ "明細情報の読み込み内容の"
										+ (r + 1) + "行目の"
										+ (c + 1) + "列目が一致しません。"
										);
								System.out.flush();
								result = false;
							}
						}
					}
				}
			}
		}
		return result;
	}

	/** 出力情報のチェック手順.
	 * <BR>
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * と
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で検知した不正データの情報が一致することを確認している。
	 *
	 * @param invalidCellInfosA 読み込んだ不正データ情報の一方
	 * @param invalidCellInfosB 読み込んだ不正データ情報のもう一方
	 * @return
	 * 不正データ情報の内容が一致していれば true
	 * を、そうでなければ false を返す。
	 */
	private boolean checkInvalidCellInfos(
						final List<InvalidCellInfo>	invalidCellInfosA,
						final List<InvalidCellInfo>	invalidCellInfosB) {
		boolean result = true;
		if (invalidCellInfosA != null && invalidCellInfosB != null) {
			if (invalidCellInfosA.size() != invalidCellInfosB.size()) {
				System.out.println(
						"!!!ERROR!!! "
						+ "不正データ情報の読み込み内容の個数が"
						+ "不一致です。"
						);
				System.out.flush();
				result = false;
			}
			else {
				for (int i = 0; i < invalidCellInfosA.size(); ++i) {
					InvalidCellInfo iciA = invalidCellInfosA.get(i);
					InvalidCellInfo iciB = invalidCellInfosB.get(i);
					if (iciA.getRowNo() != iciB.getRowNo()) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の行番号が不一致です。"
								);
						System.out.flush();
						result = false;
					}
					if (iciA.getColumnNo() != iciB.getColumnNo()) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の列番号が不一致です。"
								);
						System.out.flush();
						result = false;
					}
					if (!iciA.getName().equals(iciB.getName())) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の項目名が不一致です。"
								);
						System.out.flush();
						result = false;
					}
					if (!iciA.getMessageId()
										.equals(iciB.getMessageId())) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の"
								+ "メッセージが不一致です。"
								);
						System.out.flush();
						result = false;
					}
					if (iciA.getCellType() != iciB.getCellType()) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の"
								+ "セルの型が不一致です。"
								);
						System.out.flush();
						result = false;
					}
					if (!iciA.getCellDefine().equals(iciB.getCellDefine())) {
						System.out.println(
								"!!!ERROR!!! "
								+ "不正データ情報の"
								+ (i + 1) + "個目の"
								+ "セル定義情報が不一致です。"
								);
						System.out.flush();
						result = false;
					}
				}
			}
		}
		return result;
	}

	/** 出力情報のチェック手順.
	 * <BR>
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * と
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだ結果のすべてが一致することを確認している。
	 *
	 * @param headers
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * で読み込んだヘッダ情報
	 * @param details
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * で読み込んだ明細情報
	 * @param invalidCellInfos
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * で読み込んだ不正データ情報
	 * @param headers2
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだヘッダ情報
	 * @param details2
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだ明細情報
	 * @param invalidCellInfos2
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で読み込んだ不正データ情報
	 *
	 * @see TestUsingJxl#checkHeaders(Hashtable, Hashtable)
	 * @see TestUsingJxl#checkDetails(List, List)
	 * @see TestUsingJxl#checkInvalidCellInfos(List, List)
	 */
	private void checkOutputs(
					final Hashtable<String, String>	headers,
					final List<List<String>>		details,
					final List<InvalidCellInfo>		invalidCellInfos,
					final Hashtable<String, String>	headers2,
					final List<List<String>>		details2,
					final List<InvalidCellInfo>		invalidCellInfos2) {
		if (checkHeaders(headers, headers2)
		&& checkHeaders(headers2, headers)
		&& checkHeaders(headers, getCallBackedHeads())
		&& checkHeaders(getCallBackedHeads(), headers)) {
			System.out.println("!!!OK!!! ヘッダ情報は一致しています。");
			System.out.flush();
		}
		if (checkDetails(details, details2)
		&& checkDetails(details2, getCallBackedDetails())) {
			System.out.println("!!!OK!!! 明細情報は一致しています。");
			System.out.flush();
		}
		if (checkInvalidCellInfos(invalidCellInfos, invalidCellInfos2)
		&& checkInvalidCellInfos(invalidCellInfos2,
									getCallBackedInvalidCellInfos())) {
			System.out.println("!!!OK!!! "
								+ "不正データ情報は一致しています。");
			System.out.flush();
		}
	}

	/** 出力内容の表示.
	 * <BR>
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * の戻り値と、ファイルから読み込んだ不正データ情報、ヘッダ情報、
	 * 明細情報の内容を表示する。
	 *
	 * @param retcode
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * の戻り値
	 * @param invalidCellInfos
	 * ファイルから読み込んだ不正データ情報
	 * @param headers
	 * ファイルから読み込んだヘッダ情報
	 * @param details
	 * ファイルから読み込んだ明細情報
	 * @return
	 * 引数 retcode が {@link ReturnCode#FOUND_INVALID_CELLS} だった場合は 1
	 * を、そうでなければゼロを返す。
	 */
	private int showOutputs(
							final ReturnCode retcode,
							final List<InvalidCellInfo> invalidCellInfos,
							final Hashtable<String, String> headers,
							final List<List<String>> details,
							final LocaleInfo localeInfo) {
		// 読み込みのエラー処理
		int exitCode		= 0;
		String errorMessage	= null;
		switch (retcode) {
		case NORMAL:
			// Excel ファイルを正常に読み込めた
			break;
		case ILLEGAL_ARGUMENT:
			errorMessage = "エラーコード：" + retcode.getReturnCode()
							+ "：引数が間違ってまぁす！";
			break;
		case NOT_EXL_FILE:
			errorMessage = "エラーコード：" + retcode.getReturnCode()
							+ "：Excel ファイルじゃないんじゃない？";
			break;
		case FOUND_INVALID_CELLS:
			errorMessage = "エラーコード：" + retcode.getReturnCode()
							+ "：" + invalidCellInfos.size() + "個の"
							+ "妙なセルがあったようだな…。";
			break;
		case UNSUPPORTED_CHARSET:
			errorMessage = "エラーコード：" + retcode.getReturnCode()
							+ "：日本語で入力されていないということは…。";
			break;
		default:
			errorMessage = "エラーコード：" + retcode.getReturnCode()
							+ "：あらヤダ、これ、マジありえなぁい！";
			break;
		}
		if (errorMessage != null) {
			System.out.println(errorMessage);
			if (ReturnCode.FOUND_INVALID_CELLS == retcode) {
				int i = 0;
				for (InvalidCellInfo ici : invalidCellInfos) {
					System.out.printf(
						"診断#%d「%s」： "
						+ "%d 行 の "
						+ "%d 列、"
						+ "%s型、"
						+ "名称「%s」の"
						+ "%s項目が不正です。"
						+ "読み取れた型は %s。\n",
							++i,
							ici.getMessageId(),
							ici.getRowNo(),
							ici.getColumnNo(),
							((ici.getCellDefine()
									instanceof BooleanCellDefine)
							?	"論理値"
							: (ici.getCellDefine()
									instanceof DateCellDefine)
							?	"日付"
							: (ici.getCellDefine()
									instanceof EmptyCellDefine)
							?	"空白セル固定"
							: (ici.getCellDefine()
									instanceof NumberCellDefine)
							?	"数値"
							: (ici.getCellDefine()
									instanceof StringCellDefine)
							?	"文字列"
							:	"（不明）"
							),
						ici.getName(),
						((ici.getCellDefine()
							.getLostMandatoryMessageID() == null
						|| !"".equals(ici.getCellDefine()
											.getLostMandatoryMessageID()))
						?	"必須"
						:	""),
						((ici.getCellType() == CellTypeDefine.BOOLEAN)
						?	"論理値"
						: (ici.getCellType() == CellTypeDefine.DATE)
						?	"日付"
						: (ici.getCellType() == CellTypeDefine.EMPTY)
						?	"空白セル固定"
						: (ici.getCellType() == CellTypeDefine.NUMBER)
						?	"数値"
						: (ici.getCellType() == CellTypeDefine.STRING)
						?	"文字列"
						:	"（不明）"
						)
					);
				}
			}
			exitCode = 1;
		}
		// 読み込めた内容を表示する。
		if (headers != null) {
			// ヘッダから読み込んだ内容
			System.out.printf("ヘッダ：＝{\n");
			Enumeration<String> keys = headers.keys();
			while (keys.hasMoreElements()) {
				String key = keys.nextElement();
				System.out.printf("\t< %s : %s >\n",
									key,
									("③命令番号".equals(key))
									?	ReadXlsUtil
											.numberFormat(
												headers.get(key),
												localeInfo.getLocale())
									:	headers.get(key)
									);
				System.out.flush();
			}
			System.out.printf("}\n");
			System.out.flush();
		}
		if (details != null) {
			// 明細から読み込んだ内容
			System.out.printf("明細 (%d件)：＝{\n", details.size());
			int i = 0;
			for (List<String> line : details) {
				System.out.printf("[%d][%d]：{", i++, line.size());
				for (String value : line) {
					System.out.printf(" \"%s\"", value);
				}
				System.out.printf(" }\n");
				System.out.flush();
			}
			System.out.printf("}\n");
			System.out.flush();
		}
		return exitCode;
	}

	/** ユーティリティ.
	 * <BR>
	 * ファイルの内容をメモリに読みこむ。
	 * @param xlsFilePath
	 * 読みこむファイルのパス
	 * @return
	 * ファイルの内容を読み込んだ {@link ByteArrayInputStream}
	 * @throws IOException
	 * ファイルが存在いていなかったり、
	 * 読み込み中にエラーが起きたりしたら発生する。
	 */
	private ByteArrayInputStream createByteInputStream(
											final String xlsFilePath)
													throws IOException {
		// ファイルをメモリに読み込む手順
		File xfile = new File(xlsFilePath);
		byte[] b = new byte[(int)xfile.length()];
		FileInputStream fis = new FileInputStream(xfile);
		fis.read(b); fis.close(); fis = null;
		return new ByteArrayInputStream(b, 0, b.length);
	}

	/** ユーティリティ.
	 * @param readXlsUtil
	 * {@link ReadXlsUtil} のインスタンス
	 * @param localeInfo
	 * ロケール情報 {@link LocaleInfo}
	 * @return
	 * 現在の日時を日本の年号を含めた形式の文字列で返す。
	 */
	private String getToday(final ReadXlsUtil readXlsUtil,
							final LocaleInfo localeInfo) {
		return readXlsUtil.dateToJapaneseEraName(
				new GregorianCalendar().getTime(),
				"yyyy"
				+ "(" + ReadXlsUtil.JAPANESE_ERA_NAME_PLACE + ")"
				+ "/MM/dd HH:mm:ss.SSS z(Z)",
				localeInfo.getLocale());
	}

	/** テスト手順.
	 * <OL>
	 * <LI>
	 * {@link TestUsingJxl#createByteInputStream(String)}
	 * で、Excel ファイルをメモリに読み込む。
	 * </LI>
	 * <LI>
	 * Excel ファイルを操作する依存ライブラリに対応する
	 * {@link XlsHandler}
	 * に
	 * {@link XlsHandleForJxl}
	 * を設定する。
	 * </LI>
	 * <LI>
	 * 読みこむシートの名前は "Sheet1" に設定する。
	 * </LI>
	 * <LI>
	 * ヘッダの項目を定義する。
	 * </LI>
	 * <LI>
	 * ヘッダ情報の読み込みで不正データが見つかったら処理を中断しないようにフラグを設定する。
	 * </LI>
	 * <LI>
	 * 明細データを読みこむ最初の行番号を 7 に指定する。
	 * </LI>
	 * <LI>
	 * 明細を読みこむ行数と列数を求める
	 * {@link Breaker}
	 * に
	 * {@link RowColumnBreaker}
	 * を指定する。
	 * <BR>
	 * 最初と 2 番目の列が空のセルである行が 3 行続けばそこまでを読みこむ行とする。
	 * <BR>
	 * 明細 1 件分の最初の行に入力されているセルの個数にしたがって読みこむ列数を決定する。
	 * </LI>
	 * <LI>
	 * 明細の項目を定義する。明細情報は 2 行で構成する。列の繰り返し読み込みも行う。
	 * </LI>
	 * <LI>
	 * 不正なセルの情報を取得する最大数を 123 に設定する。
	 * </LI>
	 * <LI>
	 * 読み込んだヘッダ情報、明細情報、不正データ情報を出力するための、空の
	 * {@link Hashtable}
	 * 、
	 * {@link LinkedList}&lt;{@link List}&lt;{@link String}&gt;&gt;
	 * 、
	 * {@link LinkedList}&lt;{@link InvalidCellInfo}&gt;
	 * 、を用意する。
	 * </LI>
	 * <LI>
	 * 入力に誤りがあるセルを読み込むたびにコールバックされる不正データ・コレクター
	 * {@link InvalidCellInfoCollector}
	 * には null を設定する。
	 * </LI>
	 * <LI>
	 * {@link ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * を呼び出す。
	 * </LI>
	 * <LI>
	 * {@link ReadXls#createXlsReader(java.io.InputStream, XlsHandler, boolean)}
	 * を呼び出す。
	 * </LI>
	 * <LI>
	 * ヘッダ情報だけを読み出すために
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * を呼び出す。
	 * </LI>
	 * <LI>
	 * 明細情報だけを読み出すために
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * をもう一度呼び出す。
	 * </LI>
	 * <LI>
	 * {@link TestUsingJxl#checkOutputs(Hashtable, List, List, Hashtable, List, List)}
	 * で readInputStream() と parse() の出力情報を確認する。
	 * </LI>
	 * <LI>
	 * {@link TestUsingJxl#showOutputs(ReturnCode, List, Hashtable, List, LocaleInfo)}
	 * で出力情報を表示する。
	 * </LI>
	 * </OL>
	 * @param xlsFilePath
	 * 読みこむ Excel ファイル。
	 * JExcelAPI ({@link XlsHandleForJxl})が処理できるのは拡張子が .xls
	 * の頃のファイルだけ。
	 * <BR>
	 * Apache POI ({@link XlsHandleForPoi}) は、
	 * Office Open XML (拡張子 .xlsx) も読み込める。
	 * @return
	 * {@link TestUsingJxl#checkOutputs(Hashtable, List, List, Hashtable, List, List)}
	 * が出力情報の不一致などを検知したら 1
	 * を、{@link FileNotFoundException} が発生した場合は 2
	 * を、その他の {@link IOException} が発生した場合は 3
	 * を、いずれでもなければゼロを返す。
	 * @throws IOException
	 * ファイルの読み込みで異常が起きた場合に発生する。
	 */
	private int testReadInputStream(final String xlsFilePath)
												throws IOException {
		int exitCode = 0;
		FileInputStream fis = null;
		ByteArrayInputStream bais = null;
		ReadXls readXls = null;
		// 明細を読みこむ行数と列数を求める Breaker
		LinkedList<Integer> breakingCondRows
										= new LinkedList<Integer>();
		breakingCondRows.add(1);
		breakingCondRows.add(2);
		int maxBlankRowNum = 3;
		Breaker[] breakers = new Breaker[] {
				// デフォルトの行列読み込み打ち切り動作＝全部読んじゃえ！
				null,
				// 常に 17 列分を読みこむ。
				new RowColumnBreaker(breakingCondRows,
										maxBlankRowNum,
										17, -1, 0),
				// 空セルが 13 列つづけば、その左側のセルだけを読みこむ。
				new RowColumnBreaker(breakingCondRows,
										maxBlankRowNum,
										0, 0, 13),
				// シートに入力されているセル個数に応じて読み込む。
				new RowColumnBreaker(breakingCondRows,
										maxBlankRowNum,
										0, -1, 0)
				};
		for (Breaker breaker : breakers) {
			try {
				// Excel ファイルをメモリに読み込む。
				bais = createByteInputStream(xlsFilePath);

				// ロケール情報を設定する (日本国がデフォルト)
				LocaleInfo localeInfo = new LocaleInfo();
				// OOXML 形式のファイルか否かを示すフラグ
				boolean isOOXML = xlsFilePath.toLowerCase()
												.endsWith(".xlsx");
				// Excel ファイルを操作する依存ライブラリに対応する
				// XlsHandler
				XlsHandler xlsHandler =
								(isOOXML)
								?	new XlsHandleForPoi() // use Apatche POI
								:	new XlsHandleForJxl(); // use JExcelAPI
				// 読みこむシートの名前
				String sheetName = "Sheet1";
				// ヘッダの定義
				List<HeaderDefine> headerDefines
									= new LinkedList<HeaderDefine>();
				DurationChecker dateChecker =
									new DurationChecker(
										new GregorianCalendar(2001, 0, 1)
															.getTime(),
										new GregorianCalendar(9999, 11, 31)
															.getTime(),
										"21世紀以後の日時じゃありません。");
				headerDefines.add(new HeaderDefine(2,
						new DateCellDefine(3,
								"①日付",
								"命令の発効日が指定されていません。",
								"セルが日付型ではありません。",
								dateChecker,
								"yyyy/MM"
								)));
				headerDefines.add(new HeaderDefine(2,
						new StringCellDefine(7,
								"②命令名",
								null,
								"セルが文字列型ではありません。",
								16,
								"^(登録|変更|削除)+"
											+ "(、登録|、変更|、削除)+$",
								"入力できる全角文字は"
											+	"最大 16 文字までです。",
								"命令名は、登録、変更、削除をひとつ以上"
								+ "「、」で区切って入力してください。"
								)));
				RangeChecker numberChecker
								= new RangeChecker(
										BigDecimal
											.ZERO,
										BigDecimal
											.valueOf(123456789012345D),
										"ゼロ以上、123456789012345以下の"
										+ "数値じゃありませんね。");
				headerDefines.add(new HeaderDefine(4,
						new NumberCellDefine(5,
								"③命令番号",
								"命令番号は必須な入力です。",
								"セルが数値型ではありません。",
								numberChecker,
								15,
								8,
								RoundingMode.HALF_UP,
								"命令番号は、整数部は7桁まで"
								+ "小数部は8桁の数値を入力してください。"
								)));
				// ヘッダ情報の読み込みで不正データが見つかったら
				// 処理を中断するかどうかのフラグを設定する。
				boolean stopWhenHeaderHasInvalidValue = false;
				// 明細データを読みこむ最初の行番号を指定する。
				int firstDetailRowNo = 7;
				// 明細を読みこむ行数と列数を求める Breaker
				// ※ try 節を繰り返すループで breakers 配列の要素を
				//    ひとつづつ適用している。
				// 明細の定義
				// 明細を複数行で構成する。
				List<DetailDefine> detailDefines
									= new LinkedList<DetailDefine>();
				// 明細の 1 行目
				List<CellDefine> cds = new LinkedList<CellDefine>();
				cds.add(new StringCellDefine(1,
								"④地域コード",
								"地域を指定しなけりゃダメでしょ。",
								"セルが文字列型ではありません。",
								5,
								"^([0-9]){5}$",
								"入力できる半角数字は 5 桁までです。",
								"地域は半角数字 5 桁で指定すること。"
								));
				cds.add(new StringCellDefine(3,
								"⑤都市名",
								"",
								"セルが文字列型ではありません。",
								6,
								"^([0-9A-Za-z]){0,6}$",
								"入力できる半角英数字は 6 桁までです。",
								"都市名は6桁までの半角英数字で指定する。"
								));
				cds.add(new StringCellDefine(4,
								"⑥品名",
								"品名がないのはこまります。",
								"セルが文字列型ではありません。",
								20,
								"",
								"入力できる任意の文字は 20 バイトまでです。",
								"品名は任意の文字で指定できます。"
								));
				cds.add(new NumberCellDefine(6,
								"⑦値段",
								"値段がないのも困ります。",
								"セルが数値型ではありません。",
								null,
								12,
								2,
								RoundingMode.HALF_UP,
								"値段は、整数10桁、小数2桁で丸められます。"
								));
				cds.add(new DateCellDefine(7,
								"⑧購入日付",
								null,
								"セルが日付型ではありません。"
								+ "購入日付はyyyy-MM-ddで指定します。",
								null,
								"yyyy-MM-dd HH:mm:ss.SSS"
								));
				detailDefines.add(new DetailDefine(2, 1, cds));
				// 明細の 2 行目
				cds = new LinkedList<CellDefine>();
				cds.add(new EmptyCellDefine(3));
				cds.add(new EmptyCellDefine(4));
				cds.add(new StringCellDefine(5,
								"⑨集計項目名",
								null,
								"セルが文字列型ではありません。",
								20,
								null,
								"入力できる任意の文字は 20 桁までです。",
								"集計項目名は任意の文字が入力できます。"
								));
				cds.add(new NumberCellDefine(6,
								"⑩集計値",
								"",
								"セルが数値型ではありません。",
								null,
								14,
								2,
								RoundingMode.DOWN,
								"集計値は整数12桁、小数2桁に丸められます。"
								));
				cds.add(new EmptyCellDefine(7));
				detailDefines.add(new DetailDefine(1, 1, cds));
				// 不正なセルの情報を取得する最大数
				int maxInvalidCellNum = 123;
				// 読み取った情報のプレースホルダ
				Hashtable<String, String> headers
									= new Hashtable<String, String>();
				List<List<String>> details
									= new LinkedList<List<String>>();
				List<InvalidCellInfo> invalidCellInfos
									= new LinkedList<InvalidCellInfo>();
				// 入力に誤りがあるセルを読み込むたびにコールバックされる
				// 不正データ・コレクター
				InvalidCellInfoCollector invalidCellInfoCollector = null;
				// リターンコード
				ReturnCode retcode = null;

				// ヘッダと明細を読み込む。
				retcode = ReadXls.readInputStream(
												bais,
												localeInfo,
												xlsHandler,
												isOOXML,
												sheetName,
												headerDefines,
												this, // headerCallable
												stopWhenHeaderHasInvalidValue,
												firstDetailRowNo,
												breaker,
												detailDefines,
												this, // detailCallable
												maxInvalidCellNum,
												headers,
												details,
												invalidCellInfos,
												invalidCellInfoCollector);

				// メモリに読み込んだファイルの読み出し位置を
				// ファイルの先頭にリセット
				bais.reset();
				// 不正検知の通知を受けるコールバックで不正データを
				// 記録するリストを生成する。
				List<InvalidCellInfo> invalidCellInfos2
									= new LinkedList<InvalidCellInfo>();
				// ヘッダだけを読み込む。
				Hashtable<String, String> headers2 =
										new Hashtable<String, String>();
				readXls = ReadXls.createXlsReader(bais,
													localeInfo,
													xlsHandler,
													isOOXML);
				readXls.setSheetName(sheetName);
				readXls.setHeaderDefines(headerDefines);
				readXls.setHeaderCallable(this);
				readXls.setStopWhenHeaderHasInvalidValue(
											stopWhenHeaderHasInvalidValue);
				readXls.setFirstDetailRowNo(0);
				readXls.setBreaker(null);
				readXls.setDetailDefines(null);
				readXls.setDetailCallable(null);
				readXls.setMaxInvalidCellNum(maxInvalidCellNum);
				readXls.setInvalidCellInfoCollector(
												invalidCellInfoCollector);
				retcode = readXls.parse(headers2,
										null, // details
										invalidCellInfos2);
				// DetailCallable#call(List<List<String>>) のテストの
				// ために、 this#callBackedDetails に空リストを設定する。
				setCallBackedDetails(new LinkedList<List<String>>());
				// InvalidCellInfoCollector#notifyInvalidCellInfo(...) の
				// テストのために、this#callBackedInvalidCellInfos に
				// 空リストを設定する。
				setCallBackedInvalidCellInfos(
										new LinkedList<InvalidCellInfo>());
				// 明細だけを読み込む。
				List<List<String>> details2 =
											new LinkedList<List<String>>();
				readXls.setSheetName(sheetName);
				readXls.setHeaderDefines(null);
				readXls.setHeaderCallable(null);
				readXls.setStopWhenHeaderHasInvalidValue(
											stopWhenHeaderHasInvalidValue);
				readXls.setFirstDetailRowNo(firstDetailRowNo);
				readXls.setBreaker(breaker);
				readXls.setDetailDefines(detailDefines);
				readXls.setDetailCallable(this);
				readXls.setMaxInvalidCellNum(maxInvalidCellNum);
				readXls.setInvalidCellInfoCollector(this);
				retcode = readXls.parse(null,
										details2,
										invalidCellInfos2);

				// readInputStream() と parse() のそれぞれで読み込んだ結果
				// を比較して、等しい内容になっていることを確認する。
				checkOutputs(headers, details, invalidCellInfos,
								headers2, details2, invalidCellInfos2);
				// 読み込んだ内容の出力
				exitCode = showOutputs(retcode,
										invalidCellInfos2,
										headers2,
										details2,
										readXls.getLocaleInfo());
				// 実行を終了した日時を日本の年号を含めて表示する。
				System.out.printf("!!!実施日時!!! %s\n",
									getToday(new ReadXlsUtil(),
											readXls.getLocaleInfo()));
				System.out.flush();
				System.out.printf("!!!exitcode!!! %d\n\n", exitCode);
				System.out.flush();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				exitCode = 2;
			} catch (IOException e) {
				e.printStackTrace();
				exitCode = 3;
			} finally {
				if (readXls != null) {
					readXls.close();
					readXls = null;
				}
				if (bais != null) {
					bais.close();
					bais = null;
				}
				if (fis != null) {
					fis.close();
					fis = null;
				}
			}
		}
		return exitCode;
	}

	/** テストプログラムのエントリ.
	 * <BR>
	 * コマンド引数に Excel ファイル (.xls) を指定すると、
	 * {@link readxls.ReadXls#readInputStream(java.io.InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
	 * と
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * でそれを読み込ませて、JExcelAPI を使用した Excel ファイルの読み込み機能のテストを行う。
	 * <UL>
	 * <LI>
	 * コマンド引数にファイルを指定しなければ、使用方法の説明を表示してプログラム終了コード 4 で停止する。
	 * </LI>
	 * <LI>
	 * ファイルの読み込みで
	 * {@link IOException}
	 * を捕捉すればプログラム終了コード 5 で停止する。
	 * </LI>
	 * <LI>
	 * それ以外の場合は、
	 * {@link TestUsingJxl#testReadInputStream(String)}
	 * の戻り値がプログラム終了コードになる。
	 * </LI>
	 * <LI>
	 * 正常にプログラムが終了した場合は、ゼロがプログラム終了コードになる。
	 * </LI>
	 * </UL>
	 * @param args Excel ファイル (拡張子 .xls) を指定する。
	 */
	public static void main(final String[] args) {
		int exitCode = 0;
		if (args.length < 1
		|| (!args[0].toLowerCase().endsWith(".xls")
			&& !args[0].toLowerCase().endsWith(".xlsx"))) {
			System.out.println("Usage: java TestUsingJxl .xls or .xlsx file");
			System.out.flush();
			exitCode = 4;
		}
		else {
			// test usignjxl.ReadExl#readInputStream()
			try {
				System.out.printf("!!!入力ファイル!!! %s\n", args[0]);
				exitCode = (new TestUsingJxl())
										.testReadInputStream(args[0]);
			} catch (IOException e) {
				e.printStackTrace();
				exitCode = 5;
			}
		}
		System.exit(exitCode);
	}

	/** リソース.
	 * <P>
	 * ヘッダ部の読み込みがすんだところでコールバックされたヘッダ情報。
	 * </P>
	 */
	private Hashtable<String, String> callBackedHeads = null;

	/** getter/setter.
	 * @return
	 * {@link TestUsingJxl#callBackedHeads} を返す。
	 */
	private Hashtable<String, String> getCallBackedHeads() {
		return callBackedHeads;
	}

	/** getter/setter.
	 * @param callBackedHeads
	 * {@link TestUsingJxl#callBackedHeads} に設定するハッシュテーブル
	 */
	private void setCallBackedHeads(
						final Hashtable<String, String> callBackedHeads) {
		this.callBackedHeads = callBackedHeads;
	}

	@Override
	public Hashtable<String, String> call(
								final Hashtable<String, String> headers) {
		setCallBackedHeads(headers);
		return headers;
	}

	/** リソース.
	 * <P>
	 * コールバックされた明細情報を記録しておくリスト。
	 * </P>
	 */
	private List<List<String>> callBackedDetails = null;

	/** getter/setter.
	 * @return
	 * {@link TestUsingJxl#callBackedDetails} を返す。
	 */
	private List<List<String>> getCallBackedDetails() {
		return callBackedDetails;
	}

	/** getter/setter.
	 * @param callBackedDetails
	 * {@link TestUsingJxl#callBackedDetails} に設定するリスト
	 */
	private void setCallBackedDetails(
							final List<List<String>> callBackedDetails) {
		this.callBackedDetails = callBackedDetails;
	}

	@Override
	public List<List<String>> call(final List<List<String>> detailBlock)
												throws ReadXlsException {
		if (getCallBackedDetails() != null) {
			for (List<String> detailLine : detailBlock) {
				getCallBackedDetails().add(detailLine);
			}
		}
		return detailBlock;
	}

	/** リソース.
	 * <P>
	 * 不正データを検知するたびにコールバックされた情報を保持する
	 * List&lt;{@link InvalidCellInfo}&gt;。
	 * </P>
	 */
	private List<InvalidCellInfo> callBackedInvalidCellInfos = null;

	/** getter/setter.
	 * @return
	 * {@link TestUsingJxl#callBackedInvalidCellInfos} を返す。
	 */
	private List<InvalidCellInfo> getCallBackedInvalidCellInfos() {
		return callBackedInvalidCellInfos;
	}

	/** getter/setter.
	 * @param callBackedInvalidCellInfos
	 * {@link TestUsingJxl#callBackedInvalidCellInfos} に設定するリスト
	 */
	private void setCallBackedInvalidCellInfos(
				final List<InvalidCellInfo> callBackedInvalidCellInfos) {
		this.callBackedInvalidCellInfos = callBackedInvalidCellInfos;
	}

	@Override
	public void notifyInvalidCellInfo(final int					rowNo,
										final int				columnNo,
										final CellDefine		cellDefine,
										final CellTypeDefine	cellType,
										final String			messageId)
												throws ReadXlsException {
		if (getCallBackedInvalidCellInfos() != null) {
			getCallBackedInvalidCellInfos().add(
								new InvalidCellInfo(rowNo,
													columnNo,
													cellDefine.getName(),
													messageId,
													cellType,
													cellDefine));
		}
	}

}
